/*
 * MTB_Methaphone.h
 *
 *  Created on: 06.07.2017
 *      Author: schnell-42
 */
/*
 * ==================================================================== The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must include the following acknowledgment: "This product includes software developed
 * by the Apache Software Foundation (http://www.apache.org/)." Alternately, this acknowledgment may appear in the software itself, if and wherever such
 * third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and "Apache Commons" must not be used to endorse or promote products derived from this software
 * without prior written permission. For written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache", "Apache Commons", nor may "Apache" appear in their name, without prior written permission
 * of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals on behalf of the Apache Software Foundation. For more information on the Apache
 * Software Foundation, please see <http://www.apache.org/>.
 */
#ifndef MTB_METAPHONE_H_
#define MTB_METAPHONE_H_

#include <string>
#include <algorithm>
#include <sstream>
#include "MTB_Similarity.h"
#include "Util.h"

using namespace std;




/**
 * Returns the maxCodeLen.
 *
 * @return int
 */
int getMaxCodeLen();
/**
 * Find the metaphone value of a String. This is similar to the soundex algorithm, but better at finding similar sounding words. All input is converted to
 * upper case. Limitations: Input format is expected to be a single ASCII word with only characters in the A - Z range, no punctuation or numbers.
 *
 * @param txt
 *            String to find the metaphone code for
 * @return A metaphone code corresponding to the String supplied
 */
string metaphone(string txt);
/**
 * Sets the maxCodeLen.
 *
 * @param maxCodeLen
 *            The maxCodeLen to set
 */
void setMaxCodeLen(int maxCodeLenIN);

/**
 * A class to generate phonetic code. The initial Java implementation, William B. Brogden. December, 1997 Permission given by wbrogden for code to be used
 * anywhere. "Hanging on the Metaphone" by Lawrence Philips <i>Computer Language </i> of Dec. 1990, p 39
 *
 * @author wbrogden@bga.com
 * @author bayard@generationjava.com
 * @author Tim O'Brien
 * @version $Id: Metaphone.java,v 1.1 2005/11/23 17:37:12 Administrator Exp $
 */
class MTB_Metaphone: public MTB_Similarity {

public:
	MTB_Metaphone() {
	}
	;
	~MTB_Metaphone() {
	}
	;

	string getName();
	double getRelativeValue(string o1, string o2);
	double getAbsoluteValue(string o1, string o2);

//	 double getThreshold();
//	 bool isAbsolute();
//	 bool isSimilar(string o1, string o2);
//	 void setAbsolute(bool absolute);
//	 void setThreshold(double threshold);

};

#endif /* MTB_METAPHONE_H_ */
